@@ -24,7 +24,9 @@ from macapype.pipelines.prepare import _create_avg_reorient_pipeline
 
 from macapype.nodes.prepare import average_align
 
-from macapype.nodes.surface import (keep_gcc, wrap_afni_IsoSurface)
+from macapype.nodes.surface import (keep_gcc, IsoSurface)
+
+from macapype.nodes.correct_bias import itk_debias
 
 from skullTo3d.nodes.skull import (
     mask_auto_img)
@@ -193,6 +195,21 @@ def create_skull_t1_pipe(name="skull_t1_pipe", params={}):
         skull_t1_pipe.connect(t1_fast, "restored_image",
                               t1_skull_auto_mask, "img_file")
 
+    elif "t1_skull_mask_thr" in params.keys():
+
+        # t1_skull_mask_thr
+        t1_skull_mask_thr = NodeParams(
+            interface=Threshold(),
+            params=parse_key(params, 't1_skull_mask_thr'),
+            name="t1_skull_mask_thr")
+
+        skull_t1_pipe.connect(t1_fast, "restored_image",
+                              t1_skull_mask_thr, "in_file")
+
+        skull_t1_pipe.connect(
+            inputnode, ('indiv_params', parse_key, "t1_skull_mask_thr"),
+            t1_skull_mask_thr, "indiv_params")
+
     # t1_skull_mask_binary
     t1_skull_mask_binary = pe.Node(interface=UnaryMaths(),
                                    name="t1_skull_mask_binary")
@@ -203,6 +220,9 @@ def create_skull_t1_pipe(name="skull_t1_pipe", params={}):
     if "t1_skull_auto_mask" in params.keys():
         skull_t1_pipe.connect(t1_skull_auto_mask, "mask_img_file",
                               t1_skull_mask_binary, "in_file")
+    elif "t1_skull_mask_thr" in params.keys():
+        skull_t1_pipe.connect(t1_skull_mask_thr, "out_file",
+                              t1_skull_mask_binary, "in_file")
     else:
         skull_t1_pipe.connect(t1_fast,
                               ("partial_volume_files", get_elem, 0),
@@ -259,6 +279,10 @@ def create_skull_t1_pipe(name="skull_t1_pipe", params={}):
     skull_t1_pipe.connect(t1_skull_gcc, "gcc_nii_file",
                           t1_skull_dilate, "in_file")
 
+    skull_t1_pipe.connect(
+        inputnode, ('indiv_params', parse_key, "t1_skull_dilate"),
+        t1_skull_dilate, "indiv_params")
+
     # t1_skull_t1_fill
     t1_skull_fill = pe.Node(interface=UnaryMaths(),
                             name="t1_skull_fill")
@@ -276,11 +300,13 @@ def create_skull_t1_pipe(name="skull_t1_pipe", params={}):
     skull_t1_pipe.connect(t1_skull_fill, "out_file",
                           t1_skull_erode, "in_file")
 
+    skull_t1_pipe.connect(
+        inputnode, ('indiv_params', parse_key, "t1_skull_erode"),
+        t1_skull_erode, "indiv_params")
+
     # mesh_t1_skull #######
     mesh_t1_skull = pe.Node(
-        interface=niu.Function(input_names=["nii_file"],
-                               output_names=["stl_file"],
-                               function=wrap_afni_IsoSurface),
+        IsoSurface(),
         name="mesh_t1_skull")
 
     skull_t1_pipe.connect(t1_skull_erode, "out_file",
@@ -289,7 +315,6 @@ def create_skull_t1_pipe(name="skull_t1_pipe", params={}):
     if "t1_skull_fov" in params.keys():
 
         # t1_skull_fov ####### [okey][json]
-
         t1_skull_fov = NodeParams(
             interface=RobustFOV(),
             params=parse_key(params, "t1_skull_fov"),
@@ -299,6 +324,10 @@ def create_skull_t1_pipe(name="skull_t1_pipe", params={}):
             t1_skull_erode, "out_file",
             t1_skull_fov, "in_file")
 
+        skull_t1_pipe.connect(
+            inputnode, ('indiv_params', parse_key, "t1_skull_fov"),
+            t1_skull_fov, "indiv_params")
+
         # t1_skull_clean ####### [okey]
         t1_skull_clean = pe.Node(
             interface=niu.Function(input_names=["nii_file"],
@@ -312,9 +341,7 @@ def create_skull_t1_pipe(name="skull_t1_pipe", params={}):
 
         # mesh_robustt1_skull #######
         mesh_robustt1_skull = pe.Node(
-            interface=niu.Function(input_names=["nii_file"],
-                                   output_names=["stl_file"],
-                                   function=wrap_afni_IsoSurface),
+            interface=IsoSurface(),
             name="mesh_robustt1_skull")
 
         skull_t1_pipe.connect(
@@ -382,13 +409,32 @@ def create_skull_ct_pipe(name="skull_ct_pipe", params={}):
     skull_ct_pipe.connect(inputnode, "native_T1",
                           align_ct_on_T1, "ref_file")
 
+    if "align_ct_on_T1_2" in params:
+
+        # align_ct_on_T1
+        align_ct_on_T1_2 = pe.Node(interface=RegAladin(),
+                                   name="align_ct_on_T1_2")
+
+        align_ct_on_T1_2.inputs.rig_only_flag = True
+
+        skull_ct_pipe.connect(align_ct_on_T1, 'res_file',
+                              align_ct_on_T1_2, "flo_file")
+
+        skull_ct_pipe.connect(inputnode, "native_T1",
+                              align_ct_on_T1_2, "ref_file")
+
     # align_ct_on_stereo_T1
     align_ct_on_stereo_T1 = pe.Node(
         interface=RegResample(pad_val=0.0),
         name="align_ct_on_stereo_T1")
 
-    skull_ct_pipe.connect(align_ct_on_T1, 'res_file',
-                          align_ct_on_stereo_T1, "flo_file")
+    if "align_ct_on_T1_2" in params:
+        skull_ct_pipe.connect(align_ct_on_T1_2, 'res_file',
+                              align_ct_on_stereo_T1, "flo_file")
+
+    else:
+        skull_ct_pipe.connect(align_ct_on_T1, 'res_file',
+                              align_ct_on_stereo_T1, "flo_file")
 
     skull_ct_pipe.connect(inputnode, 'native_to_stereo_trans',
                           align_ct_on_stereo_T1, "trans_file")
@@ -471,6 +517,10 @@ def create_skull_ct_pipe(name="skull_ct_pipe", params={}):
     skull_ct_pipe.connect(ct_skull_gcc, "gcc_nii_file",
                           ct_skull_dilate, "in_file")
 
+    skull_ct_pipe.connect(
+        inputnode, ("indiv_params", parse_key, "ct_skull_dilate"),
+        ct_skull_dilate, "indiv_params")
+
     # ct_skull_fill #######  [okey]
     ct_skull_fill = pe.Node(interface=UnaryMaths(),
                             name="ct_skull_fill")
@@ -488,20 +538,56 @@ def create_skull_ct_pipe(name="skull_ct_pipe", params={}):
     skull_ct_pipe.connect(ct_skull_fill, "out_file",
                           ct_skull_erode, "in_file")
 
+    skull_ct_pipe.connect(
+        inputnode, ("indiv_params", parse_key, "ct_skull_erode"),
+        ct_skull_erode, "indiv_params")
+
     # mesh_ct_skull #######
     mesh_ct_skull = pe.Node(
-        interface=niu.Function(input_names=["nii_file"],
-                               output_names=["stl_file"],
-                               function=wrap_afni_IsoSurface),
+        interface=IsoSurface(),
         name="mesh_ct_skull")
 
     skull_ct_pipe.connect(ct_skull_erode, "out_file",
                           mesh_ct_skull, "nii_file")
 
+    if "ct_skull_fov" in params.keys():
+
+        # ct_skull_fov ####### [okey][json]
+        ct_skull_fov = NodeParams(
+            interface=RobustFOV(),
+            params=parse_key(params, "ct_skull_fov"),
+            name="ct_skull_fov")
+
+        skull_ct_pipe.connect(ct_skull_erode, "out_file",
+                              ct_skull_fov, "in_file")
+
+        skull_ct_pipe.connect(
+            inputnode, ('indiv_params', parse_key, "ct_skull_fov"),
+            ct_skull_fov, "indiv_params")
+
+        # ct_skull_clean ####### [okey]
+        ct_skull_clean = pe.Node(
+            interface=niu.Function(input_names=["nii_file"],
+                                   output_names=["gcc_nii_file"],
+                                   function=keep_gcc),
+            name="ct_skull_clean")
+
+        skull_ct_pipe.connect(ct_skull_fov, "out_roi",
+                              ct_skull_clean, "nii_file")
+
+        # mesh_robustct_skull #######
+        mesh_robustct_skull = pe.Node(
+            interface=IsoSurface(),
+            name="mesh_robustct_skull")
+
+        skull_ct_pipe.connect(ct_skull_clean, "gcc_nii_file",
+                              mesh_robustct_skull, "nii_file")
+
     # creating outputnode #######
     outputnode = pe.Node(
         niu.IdentityInterface(
             fields=["stereo_ct_skull_mask",
+                    "robustct_skull_mask", "robustct_skull_stl",
                     "ct_skull_stl"]),
         name='outputnode')
 
@@ -511,6 +597,13 @@ def create_skull_ct_pipe(name="skull_ct_pipe", params={}):
     skull_ct_pipe.connect(ct_skull_erode, "out_file",
                           outputnode, "stereo_ct_skull_mask")
 
+    if "ct_skull_fov" in params.keys():
+        skull_ct_pipe.connect(ct_skull_fov, "out_roi",
+                              outputnode, "robustct_skull_mask")
+
+        skull_ct_pipe.connect(mesh_robustct_skull, "stl_file",
+                              outputnode, "robustct_skull_stl")
+
     return skull_ct_pipe
 
 ###############################################################################
@@ -589,6 +682,19 @@ def create_skull_petra_pipe(name="skull_petra_pipe", params={}):
     skull_petra_pipe.connect(inputnode, "stereo_T1",
                              align_petra_on_stereo_T1, "ref_file")
 
+    if "petra_itk_debias" in params.keys():
+
+        # Adding early petra_debias
+        petra_itk_debias = pe.Node(
+                interface=niu.Function(
+                    input_names=["img_file"],
+                    output_names=["cor_img_file", "bias_img_file"],
+                    function=itk_debias),
+                name="petra_itk_debias")
+
+        skull_petra_pipe.connect(align_petra_on_stereo_T1, "out_file",
+                                 petra_itk_debias, "img_file")
+
     # ### head mask
     # headmask_threshold
     if "petra_head_mask_thr" in params.keys():
@@ -598,8 +704,14 @@ def create_skull_petra_pipe(name="skull_petra_pipe", params={}):
             params=parse_key(params, 'petra_head_mask_thr'),
             name="petra_head_mask_thr")
 
-        skull_petra_pipe.connect(align_petra_on_stereo_T1, "out_file",
-                                 petra_head_mask_thr, "in_file")
+        if "petra_itk_debias" in params.keys():
+
+            skull_petra_pipe.connect(petra_itk_debias, "cor_img_file",
+                                     petra_head_mask_thr, "in_file")
+        else:
+
+            skull_petra_pipe.connect(align_petra_on_stereo_T1, "out_file",
+                                     petra_head_mask_thr, "in_file")
 
         skull_petra_pipe.connect(
             inputnode, ('indiv_params', parse_key, "petra_head_mask_thr"),
@@ -615,8 +727,14 @@ def create_skull_petra_pipe(name="skull_petra_pipe", params={}):
                 params=parse_key(params, "petra_head_auto_mask"),
                 name="petra_head_auto_mask")
 
-        skull_petra_pipe.connect(align_petra_on_stereo_T1, "out_file",
-                                 petra_head_auto_mask, "img_file")
+        if "petra_itk_debias" in params.keys():
+
+            skull_petra_pipe.connect(petra_itk_debias, "cor_img_file",
+                                     petra_head_auto_mask, "img_file")
+        else:
+
+            skull_petra_pipe.connect(align_petra_on_stereo_T1, "out_file",
+                                     petra_head_auto_mask, "img_file")
 
         skull_petra_pipe.connect(
             inputnode, ('indiv_params', parse_key, "petra_head_auto_mask"),
@@ -686,26 +804,20 @@ def create_skull_petra_pipe(name="skull_petra_pipe", params={}):
     petra_hmasked = pe.Node(interface=ApplyMask(),
                             name="petra_hmasked")
 
-    skull_petra_pipe.connect(align_petra_on_stereo_T1, "out_file",
-                             petra_hmasked, "in_file")
+    if "petra_itk_debias" in params.keys():
+
+        skull_petra_pipe.connect(petra_itk_debias, "cor_img_file",
+                                 petra_hmasked, "in_file")
+    else:
+
+        skull_petra_pipe.connect(align_petra_on_stereo_T1, "out_file",
+                                 petra_hmasked, "in_file")
 
     skull_petra_pipe.connect(petra_head_erode, "out_file",
                              petra_hmasked, "mask_file")
 
-    # petra_fast
-    if "petra_fast" in params.keys():
-        petra_fast = NodeParams(interface=FAST(),
-                                params=parse_key(params, "petra_fast"),
-                                name="petra_fast")
+    if "petra_debias" in params.keys():
 
-        skull_petra_pipe.connect(petra_hmasked, "out_file",
-                                 petra_fast, "in_files")
-
-        skull_petra_pipe.connect(
-            inputnode, ('indiv_params', parse_key, "petra_fast"),
-            petra_fast, "indiv_params")
-
-    elif "petra_debias" in params.keys():
         # N4 intensity normalization over T1
         petra_debias = NodeParams(N4BiasFieldCorrection(),
                                   params=parse_key(params, "petra_debias"),
@@ -717,8 +829,22 @@ def create_skull_petra_pipe(name="skull_petra_pipe", params={}):
         skull_petra_pipe.connect(
             inputnode, ('indiv_params', parse_key, "petra_debias"),
             petra_debias, "indiv_params")
+
+    # petra_fast
+    petra_fast = NodeParams(interface=FAST(),
+                            params=parse_key(params, "petra_fast"),
+                            name="petra_fast")
+
+    skull_petra_pipe.connect(
+        inputnode, ('indiv_params', parse_key, "petra_fast"),
+        petra_fast, "indiv_params")
+
+    if "petra_debias" in params.keys():
+        skull_petra_pipe.connect(petra_debias, "output_image",
+                                 petra_fast, "in_files")
     else:
-        print("Warning, no debias was applied to petra")
+        skull_petra_pipe.connect(petra_hmasked, "out_file",
+                                 petra_fast, "in_files")
 
     if "petra_skull_auto_mask" in params.keys():
 
@@ -736,6 +862,10 @@ def create_skull_petra_pipe(name="skull_petra_pipe", params={}):
         skull_petra_pipe.connect(petra_fast, "restored_image",
                                  petra_skull_auto_mask, "img_file")
 
+        skull_petra_pipe.connect(
+            inputnode, ('indiv_params', parse_key, "petra_skull_auto_mask"),
+            petra_skull_auto_mask, "indiv_params")
+
     elif "petra_skull_mask_thr" in params.keys():
 
         # petra_skull_mask_thr
@@ -820,6 +950,10 @@ def create_skull_petra_pipe(name="skull_petra_pipe", params={}):
     skull_petra_pipe.connect(petra_skull_gcc, "gcc_nii_file",
                              petra_skull_dilate, "in_file")
 
+    skull_petra_pipe.connect(
+        inputnode, ('indiv_params', parse_key, "petra_skull_dilate"),
+        petra_skull_dilate, "indiv_params")
+
     # petra_skull_fill #######  [okey]
     petra_skull_fill = pe.Node(interface=UnaryMaths(),
                                name="petra_skull_fill")
@@ -838,11 +972,13 @@ def create_skull_petra_pipe(name="skull_petra_pipe", params={}):
     skull_petra_pipe.connect(petra_skull_fill, "out_file",
                              petra_skull_erode, "in_file")
 
+    skull_petra_pipe.connect(
+        inputnode, ('indiv_params', parse_key, "petra_skull_erode"),
+        petra_skull_erode, "indiv_params")
+
     # mesh_petra_skull #######
     mesh_petra_skull = pe.Node(
-        interface=niu.Function(input_names=["nii_file"],
-                               output_names=["stl_file"],
-                               function=wrap_afni_IsoSurface),
+        interface=IsoSurface(),
         name="mesh_petra_skull")
 
     skull_petra_pipe.connect(petra_skull_erode, "out_file",
@@ -859,6 +995,10 @@ def create_skull_petra_pipe(name="skull_petra_pipe", params={}):
         skull_petra_pipe.connect(petra_skull_erode, "out_file",
                                  petra_skull_fov, "in_file")
 
+        skull_petra_pipe.connect(
+            inputnode, ('indiv_params', parse_key, "petra_skull_fov"),
+            petra_skull_fov, "indiv_params")
+
         # petra_skull_clean ####### [okey]
         petra_skull_clean = pe.Node(
             interface=niu.Function(input_names=["nii_file"],
@@ -871,9 +1011,7 @@ def create_skull_petra_pipe(name="skull_petra_pipe", params={}):
 
         # mesh_robustpetra_skull #######
         mesh_robustpetra_skull = pe.Node(
-            interface=niu.Function(input_names=["nii_file"],
-                                   output_names=["stl_file"],
-                                   function=wrap_afni_IsoSurface),
+            interface=IsoSurface(),
             name="mesh_robustpetra_skull")
 
         skull_petra_pipe.connect(petra_skull_clean, "gcc_nii_file",
